#include <stdio.h>
#include <string.h>
#include "fips202.h"

#include "pke_parameters.h"
#include "mod_q_arithmetic.h"

static uint64_t load_littleendian(const uint8_t *x, int bytes)
{
	// Takes 4 bytes and re-arranges them in little-endian representation
  int i;
  uint64_t r = x[0];
  for (i = 1; i < bytes; i++)
    r |= (uint64_t)x[i] << (8 * i);
  return r;
}

void cbd(uint16_t s[PKE_N], const uint8_t buf[PKE_POLYERRORBYTES])
{
	// Centered binomial distribution with center=0 and \mu=8.
	// It takes a byte as an input and then computes the Hamming-weight difference between the low and high 4 bits. 
	// The Hamming-weight difference is the value of one sample. 
 
  uint32_t t, d, a[4], b[4];
  int i, j;

  for (i = 0; i < PKE_N / 4; i++)
  {
    t = load_littleendian(buf + 4 * i, 4);
    d = 0;
    for (j = 0; j < 4; j++)
      d += (t >> j) & 0x11111111;

    a[0] = d & 0xf;
    b[0] = (d >> 4) & 0xf;
    a[1] = (d >> 8) & 0xf;
    b[1] = (d >> 12) & 0xf;
    a[2] = (d >> 16) & 0xf;
    b[2] = (d >> 20) & 0xf;
    a[3] = (d >> 24) & 0xf;
    b[3] = (d >> 28);

    s[4 * i + 0] = mod(a[0] - b[0]);
    s[4 * i + 1] = mod(a[1] - b[1]);
    s[4 * i + 2] = mod(a[2] - b[2]);
    s[4 * i + 3] = mod(a[3] - b[3]);
  }
}

// In this example, both secret and error are sampled from the same distribution.
// Hence, GenSecretPoly() and GenErrorPoly() are identical.
  
void GenSecretPoly(uint16_t s[PKE_N], const uint8_t seed[PKE_NOISE_SEEDBYTES])
{
	uint8_t buf[PKE_POLYERRORBYTES];

	shake128(buf, sizeof(buf), seed, PKE_NOISE_SEEDBYTES);
	cbd(s, buf);
}

void GenErrorPoly(uint16_t e[PKE_N], const uint8_t seed[PKE_NOISE_SEEDBYTES])
{
	uint8_t buf[PKE_POLYERRORBYTES];

	shake128(buf, sizeof(buf), seed, PKE_NOISE_SEEDBYTES);
	cbd(e, buf);
}

void GenUniformPoly(uint16_t a[PKE_N], const uint8_t seed[PKE_UNIFORM_SEEDBYTES])
{

	int i;
	uint8_t buf[PKE_POLYUNIFORMBYTES];

	shake128(buf, sizeof(buf), seed, PKE_UNIFORM_SEEDBYTES);

	uint16_t *pt = (uint16_t *) buf;

  for (i = 0; i < PKE_N; i++) {
    a[i] = mod(pt[i]);						// Assuming coefficients of a(x) are at most 16 bits
  }

}

void BS2POLmsg(const uint8_t bytes[PKE_MESSAGEBYTES], uint16_t message_poly[PKE_N])
{
	// It transforms a message-byte-string of length PKE_MESSAGEBYTES into an encoded message polynomial
	size_t i, j;
	uint16_t PKE_q_by2 = PKE_q/2;

	for (j = 0; j < PKE_MESSAGEBYTES; j++)
	{
		for (i = 0; i < 8; i++)
		{
			message_poly[j * 8 + i] = ((bytes[j] >> i) & 0x01);
			message_poly[j * 8 + i] = message_poly[j * 8 + i] * PKE_q_by2;		// bit 0 gets encoded as 0. bit 1 gets encoded as q/2 
		}
	}
}


void POLmsg2BS(const uint16_t message_poly[PKE_N], uint8_t bytes[PKE_MESSAGEBYTES])
{
	// It transforms a message polynomial into a byte-string of length PKE_MESSAGEBYTES.
	size_t i, j;
	uint8_t message_bit;

	uint16_t PKE_q_by4 = PKE_q/4;
	uint16_t PKE_3q_by4 = (3*PKE_q)/4;

	memset(bytes, 0, PKE_MESSAGEBYTES);

	for (j = 0; j < PKE_MESSAGEBYTES; j++)
	{
		for (i = 0; i < 8; i++)
		{
			if ((message_poly[j*8 + i]>PKE_q_by4) && (message_poly[j*8+i]<PKE_3q_by4))
				message_bit = 1;
			else
				message_bit = 0;

			bytes[j] = bytes[j] | ((message_bit & 0x01) << i);
		}
	}
}

